-- RShamanStat UI Code
--
-- handles all UI specific events
--


local revision = tonumber(string.sub("$Revision: 66 $", 12, -3))
local rss = _G.RShamanStat
if rss.revision < revision then rss.revision = revision end


local encounterFilter = "";
local selectedIndex = 0;
local selectedButton = nil;
local selectedRecord = nil;
local selectedRecordInfo = nil;

local SetEncounterFilter = function (filter)
    encounterFilter = filter

    if selectedButton then
        selectedIndex = 0;
        selectedButton.selectedTex:Hide();
        selectedButton = nil;
    end

    RShamanStatUI_UpdateRecordList()
end

local SetRecord = function(record)
    RShamanStatFrameRecordInfo.headline:SetText(record.encounterName);
    RShamanStatFrameRecordInfo.subheadline:SetText(string.format("%.2fk Healing Done", record.effectiveTotal/1000.0));

    local difficulty = GetDifficultyInfo(record.difficulty);
    local duration = string.format("%02.f minutes %02.f seconds", record.totalTime/60, record.totalTime % 60);
    if not difficulty then
        difficulty = "Unknown Difficulty"
    end

    RShamanStatFrameRecordInfo.extra:SetText(difficulty .. "\n" .. duration);
    -- RShamanStatFrameRecordInfo.content:SetText(rss.RecordToString(record));

    selectedRecord = record;
    local r = record;
    local a = rss.AnalyzeRecord(r)
    selectedRecordInfo = {
        { label = "General", header = true, },
        { label = "Healing per Second",                 value=string.format("%.2fk", a.hps / 1000.0) },
        { label = "Spellpower",                         value=string.format("%d", r.spellpower) },
        { label = "Haste",                              value=string.format("%.2f%%", r.haste) },
        { label = "Mastery Percentage",                 value=string.format("%.2f%%", r.mastery)},
        { label = "Critical Strike Chance",             value=string.format("%.2f%%", r.crit)},

        { label = "Spellpower", header = true, },
        { label = "Required Spellpower for 1% Gain",    value=string.format("%d", a.spellpowerInc)  },
        { label = "Required Intellect for 1% Gain",     value=string.format("%d", a.intellectInc)  },

        { label = string.format("Haste (to %d breakpoint)",a.hasteBrk), header = true, },
        { label = "Required Rating for 1% Gain",        value=string.format("%d", a.hasteInc) },
        { label = string.format("Below %d Breakpoint",a.hasteBrk),        value=string.format("%d", a.hasteNoBrkInc) },

        { label = "Mastery", header = true },
        { label = "Total Throughput Gain",              value=string.format("%.2f%%", a.masteryTotalThroughputGain)  },
        { label = "Required Rating for 1% Gain",        value=string.format("%d", a.masteryInc)  },
        { label = "Average Health of Targets",          value=string.format("%.2f%%", a.averageHealth*100) },

        { label = "Critical Strike", header = true },
        { label = "Total Throughput Gain",              value=string.format("%.2f%%", a.critTotalThroughputGain)  },
        { label = "Required Rating for 1% Gain",        value=string.format("%d", a.critInc) },

        { label = "Mana Regeneration", header = true, },
        { label = "600 Spirit (w/o MTT)",               value=string.format("%.2f mp5", a.spirit_mp5)  },
        { label = "600 Crit",                           value=string.format("%.2f mp5", a.crit_mp5)  },
        { label = "Rating for 1% more Regen",           value=string.format("%d spi / %d crit", a.spiritRegenInc, a.critRegenInc)},
        { label = "Expected GoWS Mana Gain",            value=string.format("%d", a.waterShieldGlyphManaGain)}

    }
    RShamanStatUI_UpdateContentData();
end

function RShamanStatUI_OnTextChanged(self)
    local text = self:GetText();

    if ( text == SEARCH ) then
        SetEncounterFilter("");
        return;
    end

    SetEncounterFilter(text);
end

function RShamanStatUI_Load()
    RShamanStatFrame.TitleText:SetText("Resto Shaman Stat Analysis");

    -- Record List Scroll Frame
    RShamanStatFrameScrollFrame.update = RShamanStatUI_UpdateRecordList;
    HybridScrollFrame_CreateButtons(RShamanStatFrameScrollFrame, "RShamanStatFrameRecordButtonTemplate", 0, -1, "TOPLEFT", "TOPLEFT", 0, -1, "TOP", "BOTTOM");
    rss.UpdateUI = function()
        RShamanStatUI_UpdateRecordList();
        RShamanStatUI_UpdateContentData();
    end

    -- Record Info Scroll Frame
    RShamanStatFrameRecordInfoContentScrollFrame.update = RShamanStatUI_UpdateContentData;
    HybridScrollFrame_CreateButtons(RShamanStatFrameRecordInfoContentScrollFrame, "RShamanStatRecordInfoButtonTemplate", 1, -2, "TOPLEFT", "TOPLEFT", 0, -3);

end

function RShamanStatUI_OnClickSummary()
    local archive = RShamanStatUI_GetFilteredArchive();
    local filter = function(r)
        return (RShamanStatArchive.filter.difficulty == 0 or RShamanStatArchive.filter.difficulty == r.difficulty);
    end
    local record =  rss.CreateAggregateRecord("Summary for '" .. encounterFilter .."'", archive, filter)
    SetRecord(record);
end

function RShamanStatUI_OnClickRecord(self, button)
    if self == selectedButton then
        return;
    end

    if selectedButton then
        selectedButton.selectedTex:Hide();
    end

    selectedIndex = self.recordIndex;
    selectedButton = self;
    selectedButton.selectedTex:Show();

    PlaySound("INTERFACESOUND_CHARWINDOWTAB");

    local archive = RShamanStatUI_GetFilteredArchive();
    local record = archive[selectedIndex];
    SetRecord(record)
end

function RShamanStatUI_GetFilteredArchive()
    local archive = {};
	archive.isGoblin = RShamanStatArchive.isGoblin;
	archive.ancestralSwiftness = RShamanStatArchive.ancestralSwiftness;
	archive.hasteBreakpoints = RShamanStatArchive.hasteBreakpoints;
    local numRecords = 0;
    for i,rec in ipairs(RShamanStatArchive) do
        if encounterFilter == "" or strfind(string.lower(rec.encounterName), string.lower(encounterFilter), 0, true) then
            archive[numRecords+1] = rec;
            numRecords = numRecords + 1;
        end
    end
    return archive
end

function RShamanStatUI_UpdateRecordList()
    local archive = RShamanStatUI_GetFilteredArchive();
    local numRecords = #archive;

    local scrollFrame = RShamanStatFrame.scrollFrame;
    local buttons = scrollFrame.buttons;
    local numButtons = #buttons;

    if selectedButton then
        selectedButton.selectedTex:Hide();
    end

    for i = 1, numButtons do
        local button = buttons[i];
        local offset = HybridScrollFrame_GetOffset(scrollFrame);
        local index = i + offset;
        if index <= numRecords then
            button:Show();
            button.encounter:SetText(index .. ". " .. archive[index].encounterName);
            button.date:SetText(date("%c", archive[index].startTime));
            button.recordIndex = index;
            if selectedIndex == index then
                selectedButton = button;
                selectedButton.selectedTex:Show();
            end
        else
            button:Hide();
        end
    end

    HybridScrollFrame_Update(scrollFrame, 41*(numRecords)+5, 330);
end

function RShamanStatUI_UpdateContentData()
    if not selectedRecordInfo then
        RShamanStatUI_OnClickSummary();
    end
    local recordInfo = selectedRecordInfo;

    local numElements = #selectedRecordInfo;

    local scrollFrame = RShamanStatFrameRecordInfo.contentScrollFrame;
    local buttons = scrollFrame.buttons;
    local numButtons = #buttons;

    for i = 1, numButtons do
        local button = buttons[i];
        local offset = HybridScrollFrame_GetOffset(scrollFrame);
        local index = i + offset;
        local info = recordInfo[index]
        button.check:Hide();
        if index <= numElements then
            if info.header then
                button.categoryLeft:Show();
                button.categoryRight:Show();
                button.categoryMiddle:Show();
                button.expandIcon:Hide();
                button.icon:Hide();
                button.highlight:SetTexture("Interface\\TokenFrame\\UI-TokenFrame-CategoryButton");
                button.highlight:SetPoint("TOPLEFT", button, "TOPLEFT", 3, -2);
                button.highlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -3, 2);
                button.label:SetFontObject("GameFontNormal");
                -- button.label:SetPoint("LEFT", 22, 0);
                button.label:SetPoint("LEFT", 11, 0);
                button.label:SetText(info.label);
                button.value:SetText("");
            else
                button.categoryLeft:Hide();
                button.categoryRight:Hide();
                button.categoryMiddle:Hide();
                button.expandIcon:Hide();
                button.icon:Hide();
                button.highlight:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight");
                button.highlight:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0);
                button.highlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0);

                if info.disabled then
                    button.label:SetFontObject("GameFontDisable");
                    button.value:SetFontObject("GameFontDisable");
                else
                    button.label:SetFontObject("GameFontHighlight");
                    button.value:SetFontObject("GameFontHighlight");
                end
                button.label:SetPoint("LEFT", 11, 0);
                button.label:SetText(info.label);
                button.value:SetText(info.value);
                button.value:SetPoint("RIGHT", -11, 0);
            end
            button:Show();
            if ( mod(index, 2) == 1 ) then
                button.stripe:Hide();
            else
                button.stripe:Show();
            end
        else
            button:Hide();
        end
    end

    HybridScrollFrame_Update(scrollFrame, 21*(numElements)+5, 330);
end