local revision = tonumber(string.sub("$Revision: 67 $", 12, -3))
local rss = _G.RShamanStat
if rss.revision < revision then rss.revision = revision end

-- Spell IDs
rss.ANCESTRAL_AWAKENING = 52752        -- the heal from AA
rss.ANCESTRAL_GUIDANCE = 114911        -- heal copy from AG
rss.ASCENDANCE = 114049				   -- the spell which triggers the Ascendance buff
rss.CHAIN_HEAL = 1064
rss.CLEANSING_WATERS = 55562		   -- heal from glyphed Purify Spirit
rss.EARTH_SHIELD = 379                 -- the heal from Earth Shield
rss.EARTHLIVING = 51945                -- the heal from Earthliving
rss.FLAMES_OF_LIFE = 137808            -- heal from glyphed Flame Shock
rss.GIFT_OF_THE_NAARU = 59547          -- spacegoat racial; fun fact: the spell/aura ID is different for every class...
rss.GLYPH_OF_HEALING_WAVE = 55533      -- heal copy from Healing Wave
rss.GREATER_HEALING_WAVE = 77472
rss.HEALING_RAIN = 73921               -- the heal from Healing Rain
rss.HEALING_STREAM = 52042             -- the actual heal applied by HST
rss.HEALING_STREAM_2T15 = 138304       -- the 'bonus' heal applied by HST with the T15 set bonus
rss.HEALING_SURGE = 8004
rss.HEALING_TIDE = 114942              -- the actual heal applied by HTT
rss.HEALING_WAVE = 331
rss.PROC_RESTORATION_INFINITE = 139195 -- Lei Shen trinket proc
rss.RESTORATIVE_MISTS = 114083         -- heal copy from Ascendance
rss.RIPTIDE = 61295
rss.SPIRIT_LINK = 98021          	   -- the actual heal applied by SLT
rss.UNLEASH_LIFE = 73685
-- Casts that do not actually heal
rss.FLAME_SHOCK = 8050			       -- the damaging spell
rss.LIGHTNING_SHIELD = 324			   -- used for GCD monitoring
rss.PURIFY_SPIRIT = 77130              -- decurse which can trigger heal if glyphed
rss.SUMMON_ANCESTRAL_GUIDANCE = 108281 -- the spell which applies the Ancestral Guidance buff 
rss.SUMMON_EARTH_SHIELD = 974		   -- the spell which applies the Earth Shield aura
rss.SUMMON_HEALING_RAIN = 73920        -- the spell which casts Healing Rain
rss.SUMMON_HEALING_STREAM = 5394       -- the spell which summons HST
rss.SUMMON_HEALING_TIDE = 108280       -- the spell which summons HTT
rss.SUMMON_SPIRIT_LINK = 98008         -- the spell which summons SLT
rss.UNLEASH_ELEMENTS = 73680           -- the spell which applies the Unleash Life heal
-- Spells that grant Mana: logged under SPELL_ENERGIZE
rss.RESURGENCE = 101033                -- procced from crits and t15p2
rss.PROC_WATER_SHIELD = 52128          -- the on-hit effect of Water Shield
-- Aura IDs
rss.AURA_ANCESTRAL_GUIDANCE = rss.SUMMON_ANCESTRAL_GUIDANCE   -- ID of the aura AND the spell that applies it
rss.AURA_ASCENDANCE = 114052
rss.AURA_EARTH_SHIELD = 974   
rss.AURA_EARTHLIVING = rss.EARTHLIVING 
rss.AURA_GIFT_OF_THE_NAARU = rss.GIFT_OF_THE_NAARU
rss.AURA_RIPTIDE = rss.RIPTIDE      
rss.AURA_TIDAL_WAVES = 53390
rss.AURA_WATER_SHIELD = 52127
-- Haste buff auras from other classes/specs
rss.AURA_ELEMENTAL_OATH = 51470
rss.AURA_ENERGIZING_SPORES = 135678
rss.AURA_MIND_QUICKENING = 49868
rss.AURA_MOONKIN_AURA = 24907
rss.HASTE_BUFFS = { rss.AURA_ELEMENTAL_OATH , rss.AURA_ENERGIZING_SPORES , rss.AURA_MIND_QUICKENING , rss.AURA_MOONKIN_AURA }


-- Consumables
rss.HEALTHSTONE = 6262                 -- Warlock cookie
rss.HEAVY_WINDWOOL_BANDAGE = 102695
rss.MASTER_HEALING_POTION = 105708
rss.WINDWOOL_BANDAGE = 102694
rss.CONSUMABLES = { rss.HEALTHSTONE , rss.MASTER_HEALING_POTION } -- unaffected by Spellpower or Mastery
rss.BANDAGES = { rss.WINDWOOL_BANDAGE , rss.HEAVY_WINDWOOL_BANDAGE } -- cannot crit

-- Spell schools
rss.SCHOOL_FIRE = 3
rss.SCHOOL_NATURE = 4

-- Spell Descriptors

rss.SPELLS =
{
	[rss.ANCESTRAL_AWAKENING] = { copy = 0.3 },
	[rss.ANCESTRAL_GUIDANCE] = { copy = 1.8 },
	[rss.ASCENDANCE] = { aura = rss.AURA_ASCENDANCE },
	[rss.CHAIN_HEAL] = { ["SPELL_HEAL"] = { 7086, 8094, 0.6876 } , elTrigger = true, resurgence = 8849*0.333 },
	[rss.CLEANSING_WATERS] = { alias = rss.PURIFY_SPIRIT , noInt = true , noAG = true },
	[rss.EARTH_SHIELD] = { ["SPELL_HEAL"] = { 2043, 2043, 0.13 } , charges = 9 , aura = rss.AURA_EARTH_SHIELD },
	[rss.EARTHLIVING] = { ["SPELL_PERIODIC_HEAL"] = { 4376, 4376, 0.3984 } , aura = rss.AURA_EARTHLIVING , ticks = { duration = 12, tick = 3 } },
	[rss.FLAMES_OF_LIFE] = { school = rss.SCHOOL_FIRE, noMastery = true, noAG = true, alias = rss.FLAME_SHOCK ,
							["SPELL_DAMAGE"] = { 1086, 1086, 0.449 } , ["SPELL_PERIODIC_DAMAGE"] = { 2910, 2910, 2.10 } },
	[rss.FLAME_SHOCK] = { },
	[rss.GIFT_OF_THE_NAARU] = { aura = rss.AURA_GIFT_OF_THE_NAARU , ticks = { duration = 15 , tick = 3 } , noInt = true, noMastery = true, noHaste = true },
	[rss.GLYPH_OF_HEALING_WAVE] = { alias = rss.HEALING_WAVE , copy = 0.2 , noAG = true },
	[rss.GREATER_HEALING_WAVE] = { ["SPELL_HEAL"] = { 14172, 16190, 1.377 } , aaTrigger = true, elTrigger = true, resurgence = 8849*0.6 },
	[rss.HEALING_RAIN] = { ["SPELL_HEAL"] = { 2380, 2829, 0.2364 } , alias = rss.SUMMON_HEALING_RAIN , elTrigger = true, ticks = { duration = 10 , tick = 2 } },
	[rss.HEALING_STREAM] = { ["SPELL_PERIODIC_HEAL"] = { 0, 0, 0.44 } , totem = rss.SUMMON_HEALING_STREAM , ticks = { duration = 15 , tick = 2 } },
	[rss.HEALING_STREAM_2T15] = { ["SPELL_PERIODIC_HEAL"] = { 0, 0, 0.11 } , alias = rss.HEALING_STREAM , totem = rss.SUMMON_HEALING_STREAM , ticks = { duration = 15 , tick = 2 } },
	[rss.HEALING_SURGE] = { ["SPELL_HEAL"] = { 11687, 13351, 1.135 }, aaTrigger = true, elTrigger = true, resurgence = 8849 },
	[rss.HEALING_TIDE] = { ["SPELL_HEAL"] = { 4932, 4932, 0.484 } , totem = rss.SUMMON_HEALING_TIDE , ticks = { duration = 10, tick = 2 } },
	[rss.HEALING_WAVE] = { ["SPELL_HEAL"] = { 7790, 8899, 0.756 } , aaTrigger = true, elTrigger = true, resurgence = 8849 },
	[rss.PROC_RESTORATION_INFINITE] = { proc = true , noMastery = true , ["SPELL_HEAL"] = { 0, 0, 1.25 } },
	[rss.PURIFY_SPIRIT] = { },
	[rss.RESTORATIVE_MISTS] = { aura = rss.AURA_ASCENDANCE , copy = 1.0 , noAG = true },
	[rss.RIPTIDE] = { ["SPELL_HEAL"] = { 3735, 3735, 0.339 } , ["SPELL_PERIODIC_HEAL"] = { 10584, 10584, 0.96 } , 
					aura = rss.AURA_RIPTIDE , ticks = { duration = 18 , tick = 3 } , aaTrigger = true, elTrigger = true, resurgence = 8849*0.6 },
	[rss.SPIRIT_LINK] = { noMastery = true, noInt = true, noCrit = true , noAG = true },
	[rss.SUMMON_ANCESTRAL_GUIDANCE] = { },
	[rss.SUMMON_EARTH_SHIELD] = { },
	[rss.SUMMON_HEALING_RAIN] = { },
	[rss.SUMMON_HEALING_STREAM] = { },
	[rss.SUMMON_HEALING_TIDE] = { },
	[rss.SUMMON_SPIRIT_LINK] = { },
	[rss.UNLEASH_ELEMENTS] = { aaTrigger = true },
	[rss.UNLEASH_LIFE] = { ["SPELL_HEAL"] = { 3028, 3280, 0.286 } , alias = rss.UNLEASH_ELEMENTS , aaTrigger = true, elTrigger = true, resurgence = 8849*0.6 },
}

-- Buffs/Auras descriptors

rss.AURAS = {
	[rss.AURA_ASCENDANCE] = { cast = rss.ASCENDANCE , selfTarget = true },
	[rss.AURA_ANCESTRAL_GUIDANCE] = { cast = rss.SUMMON_ANCESTRAL_GUIDANCE , selfTarget = true },
	[rss.AURA_EARTH_SHIELD] = { cast = rss.SUMMON_EARTH_SHIELD , noHaste , charges = rss.SPELLS[rss.EARTH_SHIELD].charges },
	[rss.AURA_EARTHLIVING] = { ticks = rss.SPELLS[rss.EARTHLIVING].ticks },
	[rss.AURA_GIFT_OF_THE_NAARU] = { cast = rss.GIFT_OF_THE_NAARU , noHaste , ticks = rss.SPELLS[rss.GIFT_OF_THE_NAARU].ticks },
	[rss.AURA_RIPTIDE] = { cast = rss.RIPTIDE , ticks = rss.SPELLS[rss.RIPTIDE].ticks },
}

-- Stat conversion constants

rss.CRITICAL_MULTIPLIER = 2.00
rss.INT_TO_CRIT = 2533.66			-- Intellect to Crit%
rss.RATING_TO_CRIT = 600.0			-- Crit Rating to Crit%
rss.RATING_TO_MASTERY = 200.0		-- Mastery Rating to Mastery%
rss.RATING_TO_HASTE = 425.0         -- Haste Rating to Haste% (buffs and talents not included)

-- Steering constants
rss.WINDOW_PAST = 6.0               -- seconds before cast
rss.WINDOW_FUTURE = 12.0			-- seconds after cast
rss.WINDOW_RATE = 12.0              -- updates (frames) per second
rss.WINDOW_BINS = (rss.WINDOW_PAST+rss.WINDOW_FUTURE) * rss.WINDOW_RATE
rss.CAST_TIMEOUT = 15.0				-- time in seconds for a sent cast to succeed before being discarded

-- Flags for internal processing of combat events; number is the order in which events with the same timestamp are processed
rss.EVENTS = { }
rss.EVENTS.SPELL_CAST_SUCCESS = 1
rss.EVENTS.SPELL_SUMMON = 2
rss.EVENTS.SPELL_HEAL = 3
rss.EVENTS.SPELL_AURA_APPLIED = 4
rss.EVENTS.SPELL_AURA_REFRESH = 5
rss.EVENTS.SPELL_AURA_REFRESH_ON = 5
rss.EVENTS.SPELL_PERIODIC_HEAL = 6
rss.EVENTS.SPELL_AURA_REMOVED = 7
rss.EVENTS.SPELL_AURA_REFRESH_OFF = 8
rss.EVENTS.UNIT_DESTROYED = 9
rss.EVENTS.UNKNOWN = 100

rss.EVENT_CODES = {
	["SPELL_CAST_SUCCESS"] = rss.EVENTS.SPELL_CAST_SUCCESS,
	["SPELL_SUMMON"] = rss.EVENTS.SPELL_SUMMON,
	["SPELL_AURA_APPLIED"] = rss.EVENTS.SPELL_AURA_APPLIED,
	["SPELL_AURA_REFRESH"] = rss.EVENTS.SPELL_AURA_REFRESH,
	["SPELL_HEAL"] = rss.EVENTS.SPELL_HEAL,
	["SPELL_PERIODIC_HEAL"] = rss.EVENTS.SPELL_PERIODIC_HEAL,
	["SPELL_AURA_REMOVED"] = rss.EVENTS.SPELL_AURA_REMOVED,
	["UNIT_DESTROYED"] = rss.EVENTS.UNIT_DESTROYED,
}

rss.PLAYER_TOTEM_FLAGS = 8465 -- 0x2111, the flag for a totem we own

-- Talents

rss.TALENT_ANCESTRAL_SWIFTNESS = 11  -- index of the talent in the talent pane

-- Items

-- Gems that increase critical effect; only MoP gems that could be sensibly used by healers are included
rss.BURNING_PRIMAL_DIAMOND = 76885
rss.REVITALIZING_PRIMAL_DIAMOND = 76888
rss.CRITICAL_BONUS_GEMS = { rss.BURNING_PRIMAL_DIAMOND, rss.REVITALIZING_PRIMAL_DIAMOND }

-- T15 tier pieces 
rss.SET_BONUS_T15 = { ["ChestSlot"] = { 95940 , 95310 , 96684 } , 
					  ["HandsSlot"] = { 95941 , 95311 , 96685 } , 
					  ["HeadSlot"] = { 95942 , 95312 , 96686 } ,
					  ["LegsSlot"] = { 95943 , 95313 , 96687 } , 
					  ["ShoulderSlot"] = { 95944 , 95314 , 96688 } }
					  
rss.SET_BONUS_T15_CHANCE = 0.5;
rss.SET_BONUS_T15_FRACTION = 0.5;
rss.SET_BONUS_T15_RATIO = rss.SET_BONUS_T15_FRACTION/rss.SPELLS[rss.ANCESTRAL_AWAKENING].copy;
					
-- Lei-Shen trinket
rss.LIGHTNING_IMBUED_CHALICE = { ["Trinket0Slot"] = { 95817, 94530, 96189, 96561, 96933 } ,
                                 ["Trinket1Slot"] = { 95817, 94530, 96189, 96561, 96933 } }

rss.CHALICE_BASE_HEAL =
{
	[502] = 46561,
	[506] = 48329,
	[510] = 50164,
	[522] = 56071,
	[526] = 58200,
	[528] = 56286,
	[530] = 60410,
	[532] = 58423,
	[535] = 63294,
	[536] = 60641,
	[539] = 65697,
	[541] = 63534,
	[543] = 68192,
	[545] = 65946,
	[549] = 68450
}

-- Water Shield gains
rss.WATER_SHIELD = {
    mp5 = 2138,
    glyphed_mp5 = 1817.3,
    proc = 2928,
    glyphed_proc = 4392
}
